import 'dart:async';
import 'dart:io';
import 'package:auto_orientation/auto_orientation.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_mylibrary/flutter_mylibrary.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:video_player/video_player.dart';

import 'video_widget.dart';

enum EasyVideoSizeType{
  //默认显示比例
  SCREEN_TYPE_DEFAULT,
  //16:9
  SCREEN_TYPE_16_9,
  //4:3
  SCREEN_TYPE_4_3,
  //18:9
  SCREEN_TYPE_18_9,
  //全屏拉伸显示
  SCREEN_MATCH_FULL,
  //自定义比例
  SCREEN_TYPE_CUSTOM
}


enum EasyVideoState{
  UnInitialize,
  Ready,
  Playing,
  Buffering,
  Pause,
  Error,
  End
}


class EasyVideoController extends ChangeNotifier{


  VideoPlayerController _videoPlayerController;

  VideoPlayerController get videoPlayerController => _videoPlayerController;

  EasyVideoSizeType easyVideoSizeType;

  double aspectRatio;

  String cover;

  bool autoPlay;

  bool isFullScreen = false;

  ///视频尺寸
  ValueNotifier<double> videoAspectRatioNotifier = ValueNotifier(0);

  ///是否显示控制控件
  ValueNotifier<bool> showVideoStateNotifier = ValueNotifier(true);

  Duration position = Duration(seconds: 0);

  Duration duration = Duration(seconds: 0);

  ///播放状态
  EasyVideoState easyVideoState = EasyVideoState.UnInitialize;

  ValueNotifier<VideoPlayerController> videoPlayerControllerNotifier = ValueNotifier(null);

  bool isPlaying = false;


  ///倒计时用于控制控件隐藏时间
  Timer _timer;

  bool isSmoothing = false;

  int _hideStateWidgetSeconds = 3;

  int _counter = 0;

  String url;
  String asset;
  File file;
  String package;
  VideoFormat videoFormat;
  Map<String, String> httpHeaders = const {};


  EasyVideoController({
    this.url,
    this.asset,
    this.file,
    this.videoFormat,
    this.httpHeaders = const {},
    this.package,
    this.autoPlay = false,
    this.easyVideoSizeType,
    this.aspectRatio,
    this.cover
  }){
    doInitialize();
  }




  void doInitialize(){
    if(isNotEmpty(url)){
      _videoPlayerController = VideoPlayerController.network(
        url,
        formatHint: videoFormat,
        httpHeaders: httpHeaders,
      );
    }else if(isNotEmpty(asset)){
      _videoPlayerController = VideoPlayerController.asset(
          asset,
          package: package
      );
    }else if(isNotEmpty(file)){
      _videoPlayerController = VideoPlayerController.file(
        file,
      );
    }



    if(easyVideoSizeType == EasyVideoSizeType.SCREEN_TYPE_16_9){
      videoAspectRatioNotifier.value = 16 / 9;
    }else if(easyVideoSizeType == EasyVideoSizeType.SCREEN_TYPE_4_3){
      videoAspectRatioNotifier.value = 4 / 3;
    }else if(easyVideoSizeType == EasyVideoSizeType.SCREEN_TYPE_18_9){
      videoAspectRatioNotifier.value = 18 / 9;
    }else if(easyVideoSizeType == EasyVideoSizeType.SCREEN_TYPE_CUSTOM){
      videoAspectRatioNotifier.value = aspectRatio;
    }

    _videoPlayerController..initialize().then((_) {

      if(videoAspectRatioNotifier.value == 0){
        videoAspectRatioNotifier.value = _videoPlayerController.value.aspectRatio;
      }
      duration = _videoPlayerController.value.duration;

      easyVideoState = EasyVideoState.Ready;
      position = Duration(seconds: 0);
      notifyListeners();

      _videoPlayerController.addListener(() {
        VideoPlayerValue videoPlayerValue = _videoPlayerController.value;

        position = videoPlayerValue.position;

        duration = videoPlayerValue.duration;


        if(!videoPlayerValue.hasError){
          if(position.inMicroseconds > 0){
            if(position.inMicroseconds ==  duration.inMicroseconds){
              easyVideoState = EasyVideoState.End;
              doInitialize();
            }else{
              if(videoPlayerValue.isBuffering){
                easyVideoState = EasyVideoState.Buffering;
              }else if(videoPlayerValue.isPlaying){
                if(position.inMicroseconds == 0){
                  easyVideoState = EasyVideoState.Buffering;
                }else{
                  easyVideoState = EasyVideoState.Playing;
                }
              }else{
                easyVideoState = EasyVideoState.Pause;
              }
            }
          }
        }else{
          easyVideoState = EasyVideoState.Error;
          doInitialize();
        }

        notifyListeners();
      });
    });

    videoPlayerControllerNotifier.value = _videoPlayerController;
  }

  void play(){
    videoPlayerController?.play();
  }


  void pause(){
    videoPlayerController?.pause();
  }

  ///全屏逻辑
  void toggleFullScreen(BuildContext context) {
    if(isFullScreen){
      isFullScreen = false;
      Navigator.of(context, rootNavigator: true).pop();
      AutoOrientation.portraitAutoMode();
      ///显示状态栏，与底部虚拟操作按钮
      SystemChrome.setEnabledSystemUIOverlays(
          [SystemUiOverlay.top, SystemUiOverlay.bottom]);
    }else{
      isFullScreen = true;
      Navigator.of(context).push(CupertinoPageRoute(
        builder: (context) => Scaffold(
          body: EasyVideoControllerProvider(
            controller: this,
            child: VideoWidget(),
          ),
        )
      ));
      AutoOrientation.landscapeAutoMode();
      ///关闭状态栏，与底部虚拟操作按钮
      SystemChrome.setEnabledSystemUIOverlays([]);
    }
  }



  void cancelTimer(){
    if(_timer != null){
      _timer.cancel();
      _timer = null;
    }
  }

  void startHideStateCountDown(){
    cancelTimer();
    _counter = 0;
    ///显示状态并且处于播放中,开启倒计时让播放按钮消失
    _timer = Timer.periodic(Duration(seconds: 1), (timer) {
      if(!isPlaying){
        cancelTimer();
      }
      _counter =  ++_counter;
      if(_counter == _hideStateWidgetSeconds){
        if(isPlaying && !isSmoothing){
          hidePlayState();
        }
        cancelTimer();
      }
    });
  }



  void showPlayState(){
    if(!showVideoStateNotifier.value){
      showVideoStateNotifier.value = true;
    }
  }


  void hidePlayState(){
    if(showVideoStateNotifier.value){
      showVideoStateNotifier.value = false;
    }
  }

  void switchPlayState(){
    if(!isSmoothing && easyVideoState == EasyVideoState.Playing){
      showVideoStateNotifier.value = !showVideoStateNotifier.value;
    }
  }

  void onFinish(){
    showPlayState();
    _videoPlayerController.seekTo(Duration(seconds: 0));
    _videoPlayerController.pause();
  }


  @override
  void dispose() {
    super.dispose();
    cancelTimer();
  }


  static EasyVideoController of(BuildContext context) {
    final easyVideoControllerProvider = context
        .dependOnInheritedWidgetOfExactType<EasyVideoControllerProvider>();

    return easyVideoControllerProvider.controller;
  }
}


class EasyVideoControllerProvider extends InheritedWidget {
  const EasyVideoControllerProvider({
    Key key,
    @required this.controller,
    @required Widget child,
  }) : super(key: key, child: child);

  final EasyVideoController controller;

  @override
  bool updateShouldNotify(EasyVideoControllerProvider old) =>
      controller != old.controller;
}

